@@ -3,11 +3,11 @@ from __future__ import absolute_import
 import errno
 import logging
 from contextlib import contextmanager
-from collections import deque
 from abc import ABCMeta, abstractproperty
 
-from pyuv import Async, Pipe, Poll, UV_READABLE
+from pyuv import Pipe, Poll, UV_READABLE
 from pyuv.errno import strerror
+from six import with_metaclass
 
 from thriftworker.constants import BACKLOG_SIZE
 from thriftworker.utils.mixin import LoopMixin
@@ -23,7 +23,7 @@ def ignore_eagain(socket):
     try:
         yield
     except socket.error as exc:
-        if exc.errno not in (errno.EAGAIN, errno.EWOULDBLOCK):
+        if exc.errno not in (errno.EAGAIN, errno.EWOULDBLOCK, errno.EINVAL):
             raise
 
 
@@ -55,9 +55,7 @@ class Connections(object):
                 connection.close()
 
 
-class BaseAcceptor(LoopMixin):
-
-    __metaclass__ = ABCMeta
+class BaseAcceptor(with_metaclass(ABCMeta, LoopMixin)):
 
     Connections = Connections
 
@@ -95,7 +93,7 @@ class BaseAcceptor(LoopMixin):
 
     @property
     def active(self):
-        """Is current accepter active."""
+        """Is current acceptor active."""
         return self._poller.active
 
     def create_acceptor(self):
@@ -131,10 +129,17 @@ class BaseAcceptor(LoopMixin):
 
     @in_loop
     def start(self):
-        self._poller.start(UV_READABLE, self.create_acceptor())
+        if not self.active:
+            self._poller.start(UV_READABLE, self.create_acceptor())
 
     @in_loop
     def stop(self):
+        if self.active:
+            self._poller.stop()
+
+    @in_loop
+    def close(self):
+        """Close all resources."""
         self._poller.close()
         self._connections.close()
         self._socket.close()
@@ -144,48 +149,35 @@ class Acceptors(LoopMixin):
     """Maintain pool of acceptors. Start them when needed."""
 
     def __init__(self):
-        self._outgoing = deque()
-        self._acceptors = set()
+        self._acceptors = {}
         super(Acceptors, self).__init__()
 
     def __iter__(self):
         """Iterate over registered acceptors."""
-        return iter(self._acceptors)
+        return iter(self._acceptors.values())
 
     @cached_property
     def Acceptor(self):
         """Shortcut to :class:`thriftworker.acceptor.Acceptor` class."""
         return self.app.Acceptor
 
-    @cached_property
-    def _handle(self):
-        """Handle that should start acceptors in loop thread."""
-        outgoing = self._outgoing
-
-        def cb(handle):
-            while True:
-                try:
-                    callback = outgoing.popleft()
-                except IndexError:
-                    break
-                else:
-                    callback()
-
-        return Async(self.loop, cb)
-
     def register(self, fd, name, backlog=None):
         """Register new acceptor in pool."""
-        acceptor = self.Acceptor(name, fd, backlog=backlog)
-        self._acceptors.add(acceptor)
-        self._outgoing.append(acceptor.start)
-        self._handle.send()
+        self._acceptors[name] = self.Acceptor(name, fd, backlog=backlog)
+
+    def start_by_name(self, name):
+        acceptor = self._acceptors[name]
+        self.app.loop_container.callback(acceptor.start)
+
+    def stop_by_name(self, name):
+        acceptor = self._acceptors[name]
+        self.app.loop_container.callback(acceptor.stop)
 
     @in_loop
     def start(self):
-        self._handle.send()
+        pass
 
     @in_loop
     def stop(self):
-        self._handle.close()
-        for acceptor in self._acceptors:
-            acceptor.stop()
+        for acceptor in self._acceptors.values():
+            acceptor.close()
