@@ -4,15 +4,15 @@ import logging
 from collections import namedtuple
 from abc import ABCMeta, abstractmethod
 
+from six import with_metaclass
+
 from thriftworker.utils.mixin import LoopMixin
 from thriftworker.utils.loop import in_loop
 
 logger = logging.getLogger(__name__)
 
 
-class BaseWorker(LoopMixin):
-
-    __metaclass__ = ABCMeta
+class BaseWorker(with_metaclass(ABCMeta, LoopMixin)):
 
     #: Store request in this tuple.
     Request = namedtuple('Request', 'connection data request_id')
@@ -34,20 +34,23 @@ class BaseWorker(LoopMixin):
         return inner_callback
 
     @abstractmethod
-    def create_consumer(self, processor):
+    def create_consumer(self):
         raise NotImplementedError()
 
     def create_producer(self, service):
         """Create producer for connections."""
         Request = self.Request
+        create_callback = self.create_callback
         processor = self.app.services.create_processor(service)
-        consume = self.create_consumer(processor)
+        consume = self.create_consumer()
 
         def inner_producer(connection, data, request_id):
             """Enqueue given request to thread pool."""
-            consume(Request(connection=connection,
-                            data=data,
-                            request_id=request_id))
+            request = Request(connection=connection, data=data,
+                              request_id=request_id)
+            task = lambda: processor(request.data)
+            callback = create_callback(request)
+            consume(task, callback)
 
         return inner_producer
 
